@@ -5,6 +5,7 @@ import numpy as np
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import astropy.units as u
+from astropy.visualization.wcsaxes import WCSAxes
 import sunpy.visualization.wcsaxes_compat as wcsaxes_compat
 try:
     from sunpy.visualization.animator import ImageAnimator, ImageAnimatorWCS, LineAnimator
@@ -12,13 +13,14 @@ except ImportError:
     from sunpy.visualization.imageanimator import ImageAnimator, ImageAnimatorWCS, LineAnimator
 
 from ndcube import utils
+from ndcube.utils.cube import _get_extra_coord_edges
 from ndcube.mixins import sequence_plotting
 
 __all__ = ['NDCubePlotMixin']
 
 INVALID_UNIT_SET_MESSAGE = "Can only set unit for axis if corresponding coordinates in " + \
-  "axes_coordinates are set to None, an astropy Quantity or the name of an extra coord that " + \
-  "is an astropy Quantity."
+    "axes_coordinates are set to None, an astropy Quantity or the name of an extra coord that " + \
+    "is an astropy Quantity."
 
 
 class NDCubePlotMixin:
@@ -30,11 +32,11 @@ class NDCubePlotMixin:
              axes_units=None, data_unit=None, **kwargs):
         """
         Plots an interactive visualization of this cube with a slider
-        controlling the wavelength axis for data having dimensions greater than 2.
-        Plots an x-y graph onto the current axes for 2D or 1D data.
-        Keyword arguments are passed on to matplotlib.
-        Parameters other than data and wcs are passed to ImageAnimatorWCS,
-        which in turn passes them to imshow for data greater than 2D.
+        controlling the wavelength axis for data having dimensions greater than
+        2. Plots an x-y graph onto the current axes for 2D or 1D data. Keyword
+        arguments are passed on to matplotlib. Parameters other than data and
+        wcs are passed to ImageAnimatorWCS, which in turn passes them to imshow
+        for data greater than 2D.
 
         Parameters
         ----------
@@ -60,20 +62,26 @@ class NDCubePlotMixin:
             same length as the axis which will provide all values for that slider.
             If None is specified for an axis then the array indices will be used
             for that axis.
-
+            The physical coordinates expected by axes_coordinates should be an array of
+            pixel_edges.
+            A str entry in axes_coordinates signifies that an extra_coord will be used
+            for the axis's coordinates.
+            The str must be a valid name of an extra_coord that corresponds to the same
+            axis to which it is applied in the plot.
         """
         # If old API is used, convert to new API.
-        plot_axis_indices, axes_coordiantes, axes_units, data_unit, kwargs = _support_101_plot_API(
+        plot_axis_indices, axes_coordinates, axes_units, data_unit, kwargs = _support_101_plot_API(
             plot_axis_indices, axes_coordinates, axes_units, data_unit, kwargs)
         # Check kwargs are in consistent formats and set default values if not done so by user.
         naxis = len(self.dimensions)
         plot_axis_indices, axes_coordinates, axes_units = sequence_plotting._prep_axes_kwargs(
             naxis, plot_axis_indices, axes_coordinates, axes_units)
-        if naxis is 1:
+        if naxis == 1:
             ax = self._plot_1D_cube(axes, axes_coordinates,
                                     axes_units, data_unit, **kwargs)
         else:
             if len(plot_axis_indices) == 1:
+
                 ax = self._animate_cube_1D(
                     plot_axis_index=plot_axis_indices[0], axes_coordinates=axes_coordinates,
                     axes_units=axes_units, data_unit=data_unit, **kwargs)
@@ -90,14 +98,12 @@ class NDCubePlotMixin:
     def _plot_1D_cube(self, axes=None, axes_coordinates=None, axes_units=None, data_unit=None,
                       **kwargs):
         """
-        Plots a graph.
-        Keyword arguments are passed on to matplotlib.
+        Plots a graph. Keyword arguments are passed on to matplotlib.
 
         Parameters
         ----------
         data_unit: `astropy.unit.Unit`
             The data is changed to the unit given or the cube.unit if not given.
-
         """
         # Derive x-axis coordinates and unit from inputs.
         x_axis_coordinates, unit_x_axis = sequence_plotting._derive_1D_coordinates_and_units(
@@ -125,7 +131,7 @@ class NDCubePlotMixin:
             if unit_x_axis is not None:
                 raise TypeError(INVALID_UNIT_SET_MESSAGE)
         # Define default x axis label.
-        default_xlabel = "{0} [{1}]".format(xname, unit_x_axis)
+        default_xlabel = f"{xname} [{unit_x_axis}]"
         # Combine data and uncertainty with mask.
         xdata = np.ma.masked_array(xdata, self.mask)
         # Derive y-axis coordinates, uncertainty and unit from the NDCube's data.
@@ -165,8 +171,8 @@ class NDCubePlotMixin:
     def _plot_2D_cube(self, axes=None, plot_axis_indices=None, axes_coordinates=None,
                       axes_units=None, data_unit=None, **kwargs):
         """
-        Plots a 2D image onto the current
-        axes. Keyword arguments are passed on to matplotlib.
+        Plots a 2D image onto the current axes. Keyword arguments are passed on
+        to matplotlib.
 
         Parameters
         ----------
@@ -177,7 +183,6 @@ class NDCubePlotMixin:
             The first axis in WCS object will become the first axis of plot_axis_indices and
             second axis in WCS object will become the second axis of plot_axis_indices.
             Default: ['x', 'y']
-
         """
         # Set default values of kwargs if not set.
         if axes_coordinates is None:
@@ -199,17 +204,17 @@ class NDCubePlotMixin:
                 data = (self.data * self.unit).to(data_unit).value
         # Combine data with mask
         data = np.ma.masked_array(data, self.mask)
-        if axes is None:
-            try:
-                axes_coord_check = axes_coordinates == [None, None]
-            except:
-                axes_coord_check = False
-            if axes_coord_check:
+        try:
+            axes_coord_check = axes_coordinates == [None, None]
+        except Exception:
+            axes_coord_check = False
+        if axes_coord_check and (isinstance(axes, WCSAxes) or axes is None):
+            if axes is None:
                 # Build slice list for WCS for initializing WCSAxes object.
                 if self.wcs.naxis != 2:
                     slice_list = []
                     index = 0
-                    for bool_ in self.missing_axis:
+                    for bool_ in self.missing_axes:
                         if not bool_:
                             slice_list.append(axis_data[index])
                             index += 1
@@ -217,60 +222,69 @@ class NDCubePlotMixin:
                             slice_list.append(1)
                     if index != 2:
                         raise ValueError("Dimensions of WCS and data don't match")
-                    ax = wcsaxes_compat.gca_wcs(self.wcs, slices=tuple(slice_list))
+                    axes = wcsaxes_compat.gca_wcs(self.wcs, slices=tuple(slice_list))
                 else:
-                    ax = wcsaxes_compat.gca_wcs(self.wcs)
-                # Set axis labels
-                x_wcs_axis = utils.cube.data_axis_to_wcs_axis(plot_axis_indices[0],
-                                                              self.missing_axis)
-                ax.set_xlabel("{0} [{1}]".format(
-                    self.world_axis_physical_types[plot_axis_indices[0]],
-                    self.wcs.wcs.cunit[x_wcs_axis]))
-                y_wcs_axis = utils.cube.data_axis_to_wcs_axis(plot_axis_indices[1],
-                                                              self.missing_axis)
-                ax.set_ylabel("{0} [{1}]".format(
-                    self.world_axis_physical_types[plot_axis_indices[1]],
-                    self.wcs.wcs.cunit[y_wcs_axis]))
-                # Plot data
-                ax.imshow(data, **kwargs)
-            else:
-                # Else manually set axes x and y values based on user's input for axes_coordinates.
-                new_axes_coordinates, new_axis_units, default_labels = \
-                  self._derive_axes_coordinates(axes_coordinates, axes_units)
-                # Initialize axes object and set values along axis.
-                fig, ax = plt.subplots(1, 1)
-                # Since we can't assume the x-axis will be uniform, create NonUniformImage
-                # axes and add it to the axes object.
-                if plot_axis_indices[0] < plot_axis_indices[1]:
-                    data = data.transpose()
-                im_ax = mpl.image.NonUniformImage(
-                    ax, extent=(new_axes_coordinates[plot_axis_indices[0]][0],
-                                new_axes_coordinates[plot_axis_indices[0]][-1],
-                                new_axes_coordinates[plot_axis_indices[1]][0],
-                                new_axes_coordinates[plot_axis_indices[1]][-1]), **kwargs)
-                im_ax.set_data(new_axes_coordinates[plot_axis_indices[0]],
-                               new_axes_coordinates[plot_axis_indices[1]], data)
-                ax.add_image(im_ax)
-                # Set the limits, labels, etc. of the axes.
-                xlim = kwargs.pop("xlim", (new_axes_coordinates[plot_axis_indices[0]][0],
-                                           new_axes_coordinates[plot_axis_indices[0]][-1]))
-                ax.set_xlim(xlim)
-                ylim = kwargs.pop("xlim", (new_axes_coordinates[plot_axis_indices[1]][0],
-                                           new_axes_coordinates[plot_axis_indices[1]][-1]))
-                ax.set_ylim(ylim)
-                xlabel = kwargs.pop("xlabel", default_labels[plot_axis_indices[0]])
-                ylabel = kwargs.pop("ylabel", default_labels[plot_axis_indices[1]])
-                ax.set_xlabel(xlabel)
-                ax.set_ylabel(ylabel)
-        return ax
+                    axes = wcsaxes_compat.gca_wcs(self.wcs)
+
+            # Plot data
+            axes.imshow(data, **kwargs)
+
+            # Set axis labels
+            x_wcs_axis = utils.cube.data_axis_to_wcs_axis(plot_axis_indices[0],
+                                                          self.missing_axes)
+
+            axes.coords[x_wcs_axis].set_axislabel("{} [{}]".format(
+                self.world_axis_physical_types[plot_axis_indices[0]],
+                self.wcs.wcs.cunit[x_wcs_axis]))
+
+            y_wcs_axis = utils.cube.data_axis_to_wcs_axis(plot_axis_indices[1],
+                                                          self.missing_axes)
+
+            axes.coords[y_wcs_axis].set_axislabel("{} [{}]".format(
+                self.world_axis_physical_types[plot_axis_indices[1]],
+                self.wcs.wcs.cunit[y_wcs_axis]))
+
+        else:
+            # Else manually set axes x and y values based on user's input for axes_coordinates.
+            new_axes_coordinates, new_axis_units, default_labels = \
+                self._derive_axes_coordinates(axes_coordinates, axes_units, data.shape)
+            # Initialize axes object and set values along axis.
+            if axes is None:
+                axes = plt.gca()
+            # Since we can't assume the x-axis will be uniform, create NonUniformImage
+            # axes and add it to the axes object.
+            if plot_axis_indices[0] < plot_axis_indices[1]:
+                data = data.transpose()
+            im_ax = mpl.image.NonUniformImage(
+                axes, extent=(new_axes_coordinates[plot_axis_indices[0]][0],
+                              new_axes_coordinates[plot_axis_indices[0]][-1],
+                              new_axes_coordinates[plot_axis_indices[1]][0],
+                              new_axes_coordinates[plot_axis_indices[1]][-1]), **kwargs)
+            im_ax.set_data(new_axes_coordinates[plot_axis_indices[0]],
+                           new_axes_coordinates[plot_axis_indices[1]], data)
+            axes.add_image(im_ax)
+            # Set the limits, labels, etc. of the axes.
+            xlim = kwargs.pop("xlim", (new_axes_coordinates[plot_axis_indices[0]][0],
+                                       new_axes_coordinates[plot_axis_indices[0]][-1]))
+            axes.set_xlim(xlim)
+            ylim = kwargs.pop("xlim", (new_axes_coordinates[plot_axis_indices[1]][0],
+                                       new_axes_coordinates[plot_axis_indices[1]][-1]))
+            axes.set_ylim(ylim)
+
+            xlabel = kwargs.pop("xlabel", default_labels[plot_axis_indices[0]])
+            ylabel = kwargs.pop("ylabel", default_labels[plot_axis_indices[1]])
+            axes.set_xlabel(xlabel)
+            axes.set_ylabel(ylabel)
+
+        return axes
 
     def _plot_3D_cube(self, plot_axis_indices=None, axes_coordinates=None,
                       axes_units=None, data_unit=None, **kwargs):
         """
-        Plots an interactive visualization of this cube using sliders to move through axes
-        plot using in the image.
-        Parameters other than data and wcs are passed to ImageAnimatorWCS, which in turn
-        passes them to imshow.
+        Plots an interactive visualization of this cube using sliders to move
+        through axes plot using in the image. Parameters other than data and
+        wcs are passed to ImageAnimatorWCS, which in turn passes them to
+        imshow.
 
         Parameters
         ----------
@@ -290,7 +304,12 @@ class NDCubePlotMixin:
             same length as the axis which will provide all values for that slider.
             If None is specified for an axis then the array indices will be used
             for that axis.
-
+            The physical coordinates expected by axes_coordinates should be an array of
+            pixel_edges.
+            A str entry in axes_coordinates signifies that an extra_coord will be used
+            for the axis's coordinates.
+            The str must be a valid name of an extra_coord that corresponds to the same axis
+            to which it is applied in the plot.
         """
         # For convenience in inserting dummy variables later, ensure
         # plot_axis_indices are all positive.
@@ -310,16 +329,20 @@ class NDCubePlotMixin:
         data = np.ma.masked_array(data, self.mask)
         # If axes_coordinates not provided generate an ImageAnimatorWCS plot
         # using NDCube's wcs object.
+        new_axes_coordinates, new_axes_units, default_labels = \
+            self._derive_axes_coordinates(axes_coordinates, axes_units, data.shape, edges=True)
+
         if (axes_coordinates[plot_axis_indices[0]] is None and
                 axes_coordinates[plot_axis_indices[1]] is None):
+
             # If there are missing axes in WCS object, add corresponding dummy axes to data.
             if data.ndim < self.wcs.naxis:
                 new_shape = list(data.shape)
-                for i in np.arange(self.wcs.naxis)[self.missing_axis[::-1]]:
+                for i in np.arange(self.wcs.naxis)[self.missing_axes[::-1]]:
                     new_shape.insert(i, 1)
                     # Also insert dummy coordinates and units.
-                    axes_coordinates.insert(i, None)
-                    axes_units.insert(i, None)
+                    new_axes_coordinates.insert(i, None)
+                    new_axes_units.insert(i, None)
                     # Iterate plot_axis_indices if neccessary
                     for j, pai in enumerate(plot_axis_indices):
                         if pai >= i:
@@ -328,21 +351,32 @@ class NDCubePlotMixin:
                 data = data.reshape(new_shape)
             # Generate plot
             ax = ImageAnimatorWCS(data, wcs=self.wcs, image_axes=plot_axis_indices,
-                                  unit_x_axis=axes_units[plot_axis_indices[0]],
-                                  unit_y_axis=axes_units[plot_axis_indices[1]],
-                                  axis_ranges=axes_coordinates, **kwargs)
+                                  unit_x_axis=new_axes_units[plot_axis_indices[0]],
+                                  unit_y_axis=new_axes_units[plot_axis_indices[1]],
+                                  axis_ranges=new_axes_coordinates, **kwargs)
+
+            # Set the labels of the plot
+            ax.axes.coords[0].set_axislabel(
+                self.wcs.world_axis_physical_types[plot_axis_indices[0]])
+            ax.axes.coords[1].set_axislabel(
+                self.wcs.world_axis_physical_types[plot_axis_indices[1]])
+
         # If one of the plot axes is set manually, produce a basic ImageAnimator object.
         else:
-            new_axes_coordinates, new_axes_units, default_labels = \
-              self._derive_axes_coordinates(axes_coordinates, axes_units)
             # If axis labels not set by user add to kwargs.
             ax = ImageAnimator(data, image_axes=plot_axis_indices,
                                axis_ranges=new_axes_coordinates, **kwargs)
+
+            # Add the labels of the plot
+            ax.axes.set_xlabel(default_labels[plot_axis_indices[0]])
+            ax.axes.set_ylabel(default_labels[plot_axis_indices[1]])
         return ax
 
     def _animate_cube_1D(self, plot_axis_index=-1, axes_coordinates=None,
                          axes_units=None, data_unit=None, **kwargs):
-        """Animates an axis of a cube as a line plot with sliders for other axes."""
+        """
+        Animates an axis of a cube as a line plot with sliders for other axes.
+        """
         if axes_coordinates is None:
             axes_coordinates = [None] * self.data.ndim
         if axes_units is None:
@@ -350,10 +384,10 @@ class NDCubePlotMixin:
         # Get real world axis values along axis to be plotted and enter into axes_ranges kwarg.
         if axes_coordinates[plot_axis_index] is None:
             xname = self.world_axis_physical_types[plot_axis_index]
-            xdata = self.axis_world_coords(plot_axis_index)
+            xdata = self.axis_world_coords(plot_axis_index, edges=True)
         elif isinstance(axes_coordinates[plot_axis_index], str):
             xname = axes_coordinates[plot_axis_index]
-            xdata = self.extra_coords[xname]["value"]
+            xdata = _get_extra_coord_edges(self.extra_coords[xname]["value"])
         else:
             xname = ""
             xdata = axes_coordinates[plot_axis_index]
@@ -370,9 +404,18 @@ class NDCubePlotMixin:
             else:
                 unit_x_axis = None
         # Put xdata back into axes_coordinates as a masked array.
+        if len(xdata.shape) > 1:
+            # Since LineAnimator currently only accepts 1-D arrays for the x-axis, collapse xdata
+            # to single dimension by taking mean along non-plotting axes.
+            index = utils.wcs.get_dependent_data_axes(self.wcs, plot_axis_index, self.missing_axes)
+            reduce_axis = np.where(index == np.array(plot_axis_index))[0]
+
+            index = np.delete(index, reduce_axis)
+            # Reduce the data by taking mean
+            xdata = np.mean(xdata, axis=tuple(index))
         axes_coordinates[plot_axis_index] = xdata
         # Set default x label
-        default_xlabel = "{0} [{1}]".format(xname, unit_x_axis)
+        default_xlabel = f"{xname} [{unit_x_axis}]"
         # Derive y axis data
         if data_unit is None:
             data = self.data
@@ -384,16 +427,16 @@ class NDCubePlotMixin:
             else:
                 data = (self.data * self.unit).to(data_unit).value
         # Combine data with mask
-        #data = np.ma.masked_array(data, self.mask)
+        # data = np.ma.masked_array(data, self.mask)
         # Set default y label
-        default_ylabel = "Data [{0}]".format(unit_x_axis)
+        default_ylabel = f"Data [{unit_x_axis}]"
         # Initiate line animator object.
         ax = LineAnimator(data, plot_axis_index=plot_axis_index, axis_ranges=axes_coordinates,
                           xlabel=default_xlabel,
-                          ylabel="Data [{0}]".format(data_unit), **kwargs)
+                          ylabel=f"Data [{data_unit}]", **kwargs)
         return ax
 
-    def _derive_axes_coordinates(self, axes_coordinates, axes_units):
+    def _derive_axes_coordinates(self, axes_coordinates, axes_units, data_shape, edges=False):
         new_axes_coordinates = []
         new_axes_units = []
         default_labels = []
@@ -401,13 +444,28 @@ class NDCubePlotMixin:
         for i, axis_coordinate in enumerate(axes_coordinates):
             # If axis coordinate is None, derive axis values from WCS.
             if axis_coordinate is None:
-                # N.B. This assumes axes are independent.  Fix this before merging!!!
-                new_axis_coordinate = self.axis_world_coords(i)
+
+                # If the new_axis_coordinate is not independent, i.e. dimension is >2D
+                # and not equal to dimension of data, then the new_axis_coordinate must
+                # be reduced to a 1D ndarray by taking the mean along all non-plotting axes.
+                new_axis_coordinate = self.axis_world_coords(i, edges=edges)
                 axis_label_text = self.world_axis_physical_types[i]
+                # If the shape of the data is not 1, or all the axes are not dependent
+                if new_axis_coordinate.ndim != 1 and new_axis_coordinate.ndim != len(data_shape):
+                    index = utils.wcs.get_dependent_data_axes(self.wcs, i, self.missing_axes)
+                    reduce_axis = np.where(index == np.array([i]))[0]
+
+                    index = np.delete(index, reduce_axis)
+                    # Reduce the data by taking mean
+                    new_axis_coordinate = np.mean(new_axis_coordinate, axis=tuple(index))
+
             elif isinstance(axis_coordinate, str):
                 # If axis coordinate is a string, derive axis values from
                 # corresponding extra coord.
-                new_axis_coordinate = self.extra_coords[axis_coordinate]["value"]
+                # Calculate edge value if required
+                new_axis_coordinate = _get_extra_coord_edges(
+                    self.extra_coords[axis_coordinate]["value"]) if edges \
+                    else self.extra_coords[axis_coordinate]["value"]
                 axis_label_text = axis_coordinate
             else:
                 # Else user must have manually set the axis coordinates.
@@ -422,9 +480,9 @@ class NDCubePlotMixin:
                     new_axis_unit = axes_units[i]
                     new_axis_coordinate = new_axis_coordinate.to(new_axis_unit).value
             elif isinstance(new_axis_coordinate[0], datetime.datetime):
-                axis_label_text = "{0}/sec since {1}".format(
+                axis_label_text = "{}/sec since {}".format(
                     axis_label_text, new_axis_coordinate[0])
-                new_axis_coordinate = np.array([(t-new_axis_coordinate[0]).total_seconds()
+                new_axis_coordinate = np.array([(t - new_axis_coordinate[0]).total_seconds()
                                                 for t in new_axis_coordinate])
                 new_axis_unit = u.s
             else:
@@ -432,15 +490,16 @@ class NDCubePlotMixin:
                     new_axis_unit = None
                 else:
                     raise TypeError(INVALID_UNIT_SET_MESSAGE)
+
             # Derive default axis label
-            if type(new_axis_coordinate[0]) is datetime.datetime:
+            if isinstance(new_axis_coordinate, datetime.datetime):
                 if axis_label_text == default_label_text:
-                    default_label = "{0}".format(new_axis_coordinate[0].strftime("%Y/%m/%d %H:%M"))
+                    default_label = "{}".format(new_axis_coordinate.strftime("%Y/%m/%d %H:%M"))
                 else:
-                    default_label = "{0} [{1}]".format(
-                        axis_label_text, new_axis_coordinate[0].strftime("%Y/%m/%d %H:%M"))
+                    default_label = "{} [{}]".format(
+                        axis_label_text, new_axis_coordinate.strftime("%Y/%m/%d %H:%M"))
             else:
-                default_label = "{0} [{1}]".format(axis_label_text, new_axis_unit)
+                default_label = f"{axis_label_text} [{new_axis_unit}]"
             # Append new coordinates, units and labels to output list.
             new_axes_coordinates.append(new_axis_coordinate)
             new_axes_units.append(new_axis_unit)
@@ -449,7 +508,9 @@ class NDCubePlotMixin:
 
 
 def _support_101_plot_API(plot_axis_indices, axes_coordinates, axes_units, data_unit, kwargs):
-    """Check if user has used old API and convert it to new API."""
+    """
+    Check if user has used old API and convert it to new API.
+    """
     # Get old API variable values.
     image_axes = kwargs.pop("image_axes", None)
     axis_ranges = kwargs.pop("axis_ranges", None)
@@ -504,8 +565,14 @@ def _support_101_plot_API(plot_axis_indices, axes_coordinates, axes_units, data_
 
 def _raise_API_error(old_name, new_name):
     raise ValueError(
-        "Conflicting inputs: {0} (old API) cannot be set if {1} (new API) is set".format(
+        "Conflicting inputs: {} (old API) cannot be set if {} (new API) is set".format(
             old_name, new_name))
 
+
 def _raise_101_API_deprecation_warning(old_name, new_name):
-    warn("{0} is deprecated and will not be supported in version 2.0.  It will be replaced by {1}.  See docstring.".format(old_name, new_name), DeprecationWarning)
+    warn(
+        ("{} is deprecated and will not be supported in version 2.0."
+         " It will be replaced by {}. See docstring.").format(
+            old_name,
+            new_name),
+        DeprecationWarning)
