@@ -1,4 +1,5 @@
 import copy
+import numbers
 
 import numpy as np
 import matplotlib as mpl
@@ -10,20 +11,22 @@ except ImportError:
     from sunpy.visualization.imageanimator import ImageAnimatorWCS, LineAnimator
 
 from ndcube import utils
+from ndcube.utils.cube import _get_extra_coord_edges
 
 __all__ = ['NDCubeSequencePlotMixin']
 
 NON_COMPATIBLE_UNIT_MESSAGE = \
-  "All sequence sub-cubes' unit attribute are not compatible with data_unit set by user."
+    "All sequence sub-cubes' unit attribute are not compatible with data_unit set by user."
 AXES_UNIT_ERRONESLY_SET_MESSAGE = \
-  "axes_units element must be None unless corresponding axes_coordinate is None or a Quantity."
+    "axes_units element must be None unless corresponding axes_coordinate is None or a Quantity."
 
 
 class NDCubeSequencePlotMixin:
     def plot(self, axes=None, plot_axis_indices=None,
              axes_coordinates=None, axes_units=None, data_unit=None, **kwargs):
         """
-        Visualizes data in the NDCubeSequence with the sequence axis as a separate dimension.
+        Visualizes data in the NDCubeSequence with the sequence axis as a
+        separate dimension.
 
         Based on the dimensionality of the sequence and value of plot_axis_indices kwarg,
         a Line/Image Animation/Plot is produced.
@@ -61,6 +64,12 @@ class NDCubeSequencePlotMixin:
             `None` (implies derive the coordinates from the WCS objects),
             an `astropy.units.Quantity` or a `numpy.ndarray` of coordinates for each pixel,
             or a `str` denoting a valid extra coordinate.
+            The physical coordinates expected by axes_coordinates should be an array of
+            pixel_edges.
+            A str entry in axes_coordinates signifies that an extra_coord will be used for
+            the axis's coordinates.
+            The str must be a valid name of an extra_coord that corresponds to the same axis to
+            which it is applied in the plot.
 
         axes_units: `None or `list` of `None`, `astropy.units.Unit` and/or `str`
             If None units derived from the WCS objects will be used for all axes.
@@ -88,9 +97,9 @@ class NDCubeSequencePlotMixin:
 
         Returns
         -------
-        ax: `matplotlib.axes.Axes`, `ndcube.mixins.sequence_plotting.ImageAnimatorNDCubeSequence` or `ndcube.mixins.sequence_plotting.ImageAnimatorCubeLikeNDCubeSequence`
+        ax: `matplotlib.axes.Axes`, `ndcube.mixins.sequence_plotting.ImageAnimatorNDCubeSequence`
+             or `ndcube.mixins.sequence_plotting.ImageAnimatorCubeLikeNDCubeSequence`
             Axes or animation object depending on dimensionality of NDCubeSequence
-
         """
         # Check kwargs are in consistent formats and set default values if not done so by user.
         naxis = len(self.dimensions)
@@ -128,7 +137,8 @@ class NDCubeSequencePlotMixin:
     def plot_as_cube(self, axes=None, plot_axis_indices=None,
                      axes_coordinates=None, axes_units=None, data_unit=None, **kwargs):
         """
-        Visualizes data in the NDCubeSequence with the sequence axis folded into the common axis.
+        Visualizes data in the NDCubeSequence with the sequence axis folded
+        into the common axis.
 
         Based on the cube-like dimensionality of the sequence and value of plot_axis_indices
         kwarg, a Line/Image Plot/Animation is produced.
@@ -167,6 +177,12 @@ class NDCubeSequencePlotMixin:
             `None` (implies derive the coordinates from the WCS objects),
             an `astropy.units.Quantity` or a `numpy.ndarray` of coordinates for each pixel,
             or a `str` denoting a valid extra coordinate.
+            The physical coordinates expected by axes_coordinates should be an array of
+            pixel_edges.
+            A str entry in axes_coordinates signifies that an extra_coord will be used for
+            the axis's coordinates.
+            The str must be a valid name of an extra_coord that corresponds to the same axis to
+            which it is applied in the plot.
 
         axes_units: `None or `list` of `None`, `astropy.units.Unit` and/or `str`
             If None units derived from the WCS objects will be used for all axes.
@@ -194,9 +210,9 @@ class NDCubeSequencePlotMixin:
 
         Returns
         -------
-        ax: ax: `matplotlib.axes.Axes`, `ndcube.mixins.sequence_plotting.ImageAnimatorNDCubeSequence` or `ndcube.mixins.sequence_plotting.ImageAnimatorCubeLikeNDCubeSequence`
+        ax: ax: `matplotlib.axes.Axes`, `ndcube.mixins.sequence_plotting.ImageAnimatorNDCubeSequence` or
+                `ndcube.mixins.sequence_plotting.ImageAnimatorCubeLikeNDCubeSequence`
             Axes or animation object depending on dimensionality of NDCubeSequence
-
         """
         # Verify common axis is set.
         if self._common_axis is None:
@@ -253,6 +269,12 @@ class NDCubeSequencePlotMixin:
             each pixel along the x-axis.
             If a `str`, denotes the extra coordinate to be used.  The extra coordinate must
             correspond to the sequence axis.
+            The physical coordinates expected by axes_coordinates should be an array of
+            pixel_edges.
+            A str entry in axes_coordinates signifies that an extra_coord will be used for
+            the axis's coordinates.
+            The str must be a valid name of an extra_coord that corresponds to the same axis to
+            which it is applied in the plot.
 
         axes_units: `astropy.unit.Unit` or valid unit `str` or length 1 `list` of those types.
             Unit in which X-axis should be displayed.  Must be compatible with the unit of
@@ -262,7 +284,6 @@ class NDCubeSequencePlotMixin:
         data_unit: `astropy.units.unit` or valid unit `str`
             The units into which the y-axis should be displayed.  The unit attribute of all
             the sub-cubes must be compatible to set this kwarg.
-
         """
         # Derive x-axis coordinates and unit from inputs.
         x_axis_coordinates, unit_x_axis = _derive_1D_coordinates_and_units(axes_coordinates,
@@ -320,21 +341,21 @@ class NDCubeSequencePlotMixin:
                 xdata = xdata.to(unit_x_axis)
         else:
             unit_x_axis = None
-        default_xlabel = "{0} [{1}]".format(xname, unit_x_axis)
+        default_xlabel = f"{xname} [{unit_x_axis}]"
         fig, ax = _make_1D_sequence_plot(xdata, ydata, yerror, unit_y_axis, default_xlabel, kwargs)
         return ax
 
     def _plot_2D_sequence_as_1Dline(self, axes_coordinates=None,
                                     axes_units=None, data_unit=None, **kwargs):
         """
-        Visualizes an NDCubeSequence of 1D NDCubes with a common axis as a line plot.
+        Visualizes an NDCubeSequence of 1D NDCubes with a common axis as a line
+        plot.
 
         Called if plot_as_cube=True.
 
         Parameters
         ----------
         Same as _plot_1D_sequence
-
         """
         # Derive x-axis coordinates and unit from inputs.
         x_axis_coordinates, unit_x_axis = _derive_1D_coordinates_and_units(axes_coordinates,
@@ -383,7 +404,7 @@ class NDCubeSequencePlotMixin:
             if unit_x_axis is None:
                 print('b', unit_x_axis)
                 unit_x_axis = np.asarray(self[0].wcs.wcs.cunit)[
-                    np.invert(self[0].missing_axis)][0]
+                    np.invert(self[0].missing_axes)][0]
                 print('c', unit_x_axis)
             xdata = u.Quantity(np.concatenate([cube.axis_world_coords().to(unit_x_axis).value
                                                for cube in self.data]), unit=unit_x_axis)
@@ -402,7 +423,7 @@ class NDCubeSequencePlotMixin:
                 xdata = xdata.to(unit_x_axis)
         else:
             unit_x_axis = None
-        default_xlabel = "{0} [{1}]".format(xname, unit_x_axis)
+        default_xlabel = f"{xname} [{unit_x_axis}]"
         # For consistency, make xdata an array if a Quantity. Wait until now
         # because if xdata is a Quantity, its unit is needed until now.
         if isinstance(xdata, u.Quantity):
@@ -421,7 +442,6 @@ class NDCubeSequencePlotMixin:
         Parameters
         ----------
         Same as self.plot()
-
         """
         # Set default values of kwargs if not set.
         if axes_coordinates is None:
@@ -452,13 +472,13 @@ class NDCubeSequencePlotMixin:
         if axes_coordinates[cube_axis_index] is None:
             if cube_axis_unit is None:
                 cube_axis_unit = np.array(self[0].wcs.wcs.cunit)[
-                    np.invert(self[0].missing_axis)][0]
+                    np.invert(self[0].missing_axes)][0]
             cube_axis_coords = self[0].axis_world_coords().to(cube_axis_unit).value
             cube_axis_name = self.world_axis_physical_types[1]
         else:
             if isinstance(axes_coordinates[cube_axis_index], str):
                 cube_axis_coords = \
-                  self[0].extra_coords[axes_coordinates[cube_axis_index]]["value"]
+                    self[0].extra_coords[axes_coordinates[cube_axis_index]]["value"]
                 cube_axis_name = axes_coordinates[cube_axis_index]
             else:
                 cube_axis_coords = axes_coordinates[cube_axis_index]
@@ -472,7 +492,7 @@ class NDCubeSequencePlotMixin:
             else:
                 if cube_axis_unit is not None:
                     raise ValueError(AXES_UNIT_ERRONESLY_SET_MESSAGE)
-        default_cube_axis_label = "{0} [{1}]".format(cube_axis_name, cube_axis_unit)
+        default_cube_axis_label = f"{cube_axis_name} [{cube_axis_unit}]"
         axes_coordinates[cube_axis_index] = cube_axis_coords
         axes_units[cube_axis_index] = cube_axis_unit
         # Derive the coordinates, unit, and default label of the sequence axis.
@@ -482,7 +502,7 @@ class NDCubeSequencePlotMixin:
             sequence_axis_name = self.world_axis_physical_types[0]
         elif isinstance(axes_coordinates[sequence_axis_index], str):
             sequence_axis_coords = \
-              self.sequence_axis_extra_coords[axes_coordinates[sequence_axis_index]]
+                self.sequence_axis_extra_coords[axes_coordinates[sequence_axis_index]]
             sequence_axis_name = axes_coordinates[sequence_axis_index]
         else:
             sequence_axis_coords = axes_coordinates[sequence_axis_index]
@@ -496,7 +516,7 @@ class NDCubeSequencePlotMixin:
         else:
             if sequence_axis_unit is not None:
                 raise ValueError(AXES_UNIT_ERRONESLY_SET_MESSAGE)
-        default_sequence_axis_label = "{0} [{1}]".format(sequence_axis_name, sequence_axis_unit)
+        default_sequence_axis_label = f"{sequence_axis_name} [{sequence_axis_unit}]"
         axes_coordinates[sequence_axis_index] = sequence_axis_coords
         axes_units[sequence_axis_index] = sequence_axis_unit
         axes_labels = [None, None]
@@ -530,10 +550,10 @@ class NDCubeSequencePlotMixin:
                                      axes_coordinates=None, axes_units=None, data_unit=None,
                                      **kwargs):
         """
-        Visualizes an NDCubeSequence of 2D NDCubes with a common axis as a 2D image.
+        Visualizes an NDCubeSequence of 2D NDCubes with a common axis as a 2D
+        image.
 
         Called if plot_as_cube=True.
-
         """
         # Set default values of kwargs if not set.
         if axes_coordinates is None:
@@ -564,14 +584,14 @@ class NDCubeSequencePlotMixin:
         if axes_coordinates[cube_axis_index] is None:
             if cube_axis_unit is None:
                 cube_axis_unit = np.array(self[0].wcs.wcs.cunit)[
-                    np.invert(self[0].missing_axis)][0]
+                    np.invert(self[0].missing_axes)][0]
             cube_axis_coords = \
                 self[0].axis_world_coords()[cube_axis_index].to(cube_axis_unit).value
             cube_axis_name = self.cube_like_world_axis_physical_types[1]
         else:
             if isinstance(axes_coordinates[cube_axis_index], str):
                 cube_axis_coords = \
-                  self[0].extra_coords[axes_coordinates[cube_axis_index]]["value"]
+                    self[0].extra_coords[axes_coordinates[cube_axis_index]]["value"]
                 cube_axis_name = axes_coordinates[cube_axis_index]
             else:
                 cube_axis_coords = axes_coordinates[cube_axis_index]
@@ -585,7 +605,7 @@ class NDCubeSequencePlotMixin:
             else:
                 if cube_axis_unit is not None:
                     raise ValueError(AXES_UNIT_ERRONESLY_SET_MESSAGE)
-        default_cube_axis_label = "{0} [{1}]".format(cube_axis_name, cube_axis_unit)
+        default_cube_axis_label = f"{cube_axis_name} [{cube_axis_unit}]"
         axes_coordinates[cube_axis_index] = cube_axis_coords
         axes_units[cube_axis_index] = cube_axis_unit
         # Derive the coordinates, unit, and default label of the common axis.
@@ -594,7 +614,7 @@ class NDCubeSequencePlotMixin:
             # Concatenate values along common axis for each cube.
             if common_axis_unit is None:
                 wcs_common_axis_index = utils.cube.data_axis_to_wcs_axis(
-                    common_axis_index, self[0].missing_axis)
+                    common_axis_index, self[0].missing_axes)
                 common_axis_unit = np.array(self[0].wcs.wcs.cunit)[wcs_common_axis_index]
             common_axis_coords = u.Quantity(np.concatenate(
                 [cube.axis_world_coords()[common_axis_index].to(common_axis_unit).value
@@ -602,7 +622,7 @@ class NDCubeSequencePlotMixin:
             common_axis_name = self.cube_like_world_axis_physical_types[common_axis_index]
         elif isinstance(axes_coordinates[common_axis_index], str):
             common_axis_coords = \
-              self.common_axis_extra_coords[axes_coordinates[common_axis_index]]
+                self.common_axis_extra_coords[axes_coordinates[common_axis_index]]
             common_axis_name = axes_coordinates[common_axis_index]
         else:
             common_axis_coords = axes_coordinates[common_axis_index]
@@ -616,7 +636,7 @@ class NDCubeSequencePlotMixin:
         else:
             if common_axis_unit is not None:
                 raise ValueError(AXES_UNIT_ERRONESLY_SET_MESSAGE)
-        default_common_axis_label = "{0} [{1}]".format(common_axis_name, common_axis_unit)
+        default_common_axis_label = f"{common_axis_name} [{common_axis_unit}]"
         axes_coordinates[common_axis_index] = common_axis_coords
         axes_units[common_axis_index] = common_axis_unit
         axes_labels = [None, None]
@@ -682,6 +702,8 @@ class ImageAnimatorNDCubeSequence(ImageAnimatorWCS):
         same length as the axis which will provide all values for that slider.
         If None is specified for an axis then the array indices will be used
         for that axis.
+        The physical coordinates expected by axis_ranges should be an array of
+        pixel_edges.
 
     interval: `int`
         Animation interval in ms
@@ -702,8 +724,8 @@ class ImageAnimatorNDCubeSequence(ImageAnimatorWCS):
         The unit of y axis.
 
     Extra keywords are passed to imshow.
-
     """
+
     def __init__(self, seq, wcs=None, axes=None, plot_axis_indices=None,
                  axes_coordinates=None, axes_units=None, data_unit=None, **kwargs):
         self.sequence = seq.data  # Required by parent class.
@@ -727,16 +749,16 @@ class ImageAnimatorNDCubeSequence(ImageAnimatorWCS):
         # shape for an missing axes.
         if seq[0].wcs.naxis != len(seq.dimensions) - 1:
             new_shape = list(data_stack.shape)
-            for i in np.arange(seq[0].wcs.naxis)[seq[0].missing_axis[::-1]]:
-                new_shape.insert(i+1, 1)
+            for i in np.arange(seq[0].wcs.naxis)[seq[0].missing_axes[::-1]]:
+                new_shape.insert(i + 1, 1)
                 # Also insert dummy coordinates and units.
-                axes_coordinates.insert(i+1, None)
-                axes_units.insert(i+1, None)
+                axes_coordinates.insert(i + 1, None)
+                axes_units.insert(i + 1, None)
             data_stack = data_stack.reshape(new_shape)
         # Add dummy axis to WCS object to represent sequence axis.
         new_wcs = utils.wcs.append_sequence_axis_to_wcs(wcs)
 
-        super(ImageAnimatorNDCubeSequence, self).__init__(
+        super().__init__(
             data_stack, wcs=new_wcs, image_axes=plot_axis_indices, axis_ranges=axes_coordinates,
             unit_x_axis=axes_units[plot_axis_indices[0]],
             unit_y_axis=axes_units[plot_axis_indices[1]], **kwargs)
@@ -777,6 +799,8 @@ class ImageAnimatorCubeLikeNDCubeSequence(ImageAnimatorWCS):
         same length as the axis which will provide all values for that slider.
         If None is specified for an axis then the array indices will be used
         for that axis.
+        The physical coordinates expected by axis_ranges should be an array of
+        pixel_edges.
 
     interval: `int`
         Animation interval in ms
@@ -797,8 +821,8 @@ class ImageAnimatorCubeLikeNDCubeSequence(ImageAnimatorWCS):
         The unit of y axis.
 
     Extra keywords are passed to imshow.
-
     """
+
     def __init__(self, seq, wcs=None, axes=None, plot_axis_indices=None,
                  axes_coordinates=None, axes_units=None, data_unit=None, **kwargs):
         if seq._common_axis is None:
@@ -827,14 +851,14 @@ class ImageAnimatorCubeLikeNDCubeSequence(ImageAnimatorWCS):
         # shape for an missing axes.
         if seq[0].wcs.naxis != len(seq._dimensions) - 1:
             new_shape = list(data_concat.shape)
-            for i in np.arange(seq[0].wcs.naxis)[seq[0].missing_axis[::-1]]:
+            for i in np.arange(seq[0].wcs.naxis)[seq[0].missing_axes[::-1]]:
                 new_shape.insert(i, 1)
                 # Also insert dummy coordinates and units.
                 axes_coordinates.insert(i, None)
                 axes_units.insert(i, None)
             data_concat = data_concat.reshape(new_shape)
 
-        super(ImageAnimatorCubeLikeNDCubeSequence, self).__init__(
+        super().__init__(
             data_concat, wcs=wcs, image_axes=plot_axis_indices, axis_ranges=axes_coordinates,
             unit_x_axis=axes_units[plot_axis_indices[0]],
             unit_y_axis=axes_units[plot_axis_indices[1]], **kwargs)
@@ -849,7 +873,7 @@ class ImageAnimatorCubeLikeNDCubeSequence(ImageAnimatorWCS):
             val, self.cumul_cube_lengths)
         sequence_index = sequence_slice.sequence_index
         cube_index = sequence_slice.common_axis_item
-        list_slices_wcsaxes[self.wcs.naxis-ax_ind-1] = cube_index
+        list_slices_wcsaxes[self.wcs.naxis - ax_ind - 1] = cube_index
         self.slices_wcsaxes = list_slices_wcsaxes
         if val != slider.cval:
             self.axes.reset_wcs(
@@ -894,6 +918,8 @@ class LineAnimatorNDCubeSequence(LineAnimator):
         same length as the axis which will provide all values for that slider.
         If None is specified for an axis then the array indices will be used
         for that axis.
+        The physical coordinates expected by axis_ranges should be an array of
+        pixel_edges.
 
     interval: `int`
         Animation interval in ms
@@ -914,8 +940,8 @@ class LineAnimatorNDCubeSequence(LineAnimator):
         The unit of y axis.
 
     Extra keywords are passed to imshow.
-
     """
+
     def __init__(self, seq, plot_axis_index=None, axis_ranges=None, unit_x_axis=None,
                  data_unit=None, xlabel=None, ylabel=None, xlim=None, ylim=None, **kwargs):
         if plot_axis_index is None:
@@ -956,23 +982,24 @@ class LineAnimatorNDCubeSequence(LineAnimator):
         if axis_ranges is None:
             axis_ranges = [None] * len(seq.dimensions)
             if plot_axis_index == 0:
-                axis_ranges[plot_axis_index] = np.arange(len(seq.data))
+                axis_ranges[plot_axis_index] = _get_extra_coord_edges(
+                    np.arange(len(seq.data)), axis=plot_axis_index)
             else:
                 cube_plot_axis_index = plot_axis_index - 1
                 # Define unit of x-axis if not supplied by user.
                 if unit_x_axis is None:
                     wcs_plot_axis_index = utils.cube.data_axis_to_wcs_axis(
-                        cube_plot_axis_index, seq[0].missing_axis)
+                        cube_plot_axis_index, seq[0].missing_axes)
                     unit_x_axis = np.asarray(seq[0].wcs.wcs.cunit)[wcs_plot_axis_index]
                 # Get x-axis values from each cube and combine into a single
                 # array for axis_ranges kwargs.
-                x_axis_coords = _get_non_common_axis_x_axis_coords(seq.data, cube_plot_axis_index,
-                                                                   unit_x_axis)
+                x_axis_coords = _get_extra_coord_edges(_get_non_common_axis_x_axis_coords(
+                    seq.data, cube_plot_axis_index, unit_x_axis), axis=plot_axis_index)
                 axis_ranges[plot_axis_index] = np.stack(x_axis_coords)
             # Set x-axis label.
             if xlabel is None:
-                xlabel = "{0} [{1}]".format(seq.world_axis_physical_types[plot_axis_index],
-                                            unit_x_axis)
+                xlabel = "{} [{}]".format(seq.world_axis_physical_types[plot_axis_index],
+                                          unit_x_axis)
         else:
             # If the axis range is being defined by an extra coordinate...
             if isinstance(axis_ranges[plot_axis_index], str):
@@ -1004,15 +1031,15 @@ class LineAnimatorNDCubeSequence(LineAnimator):
                     if extra_coord_type.all() == extra_coord_type[0]:
                         extra_coord_type = extra_coord_type[0]
                     else:
-                        raise TypeError("Extra coord {0} must be of same type for all NDCubes to "
+                        raise TypeError("Extra coord {} must be of same type for all NDCubes to "
                                         "use it to define a plot axis.".format(axis_extra_coord))
                     if extra_coord_axes.all() == extra_coord_axes[0]:
-                        if isinstance(extra_coord_axes[0], (int, np.int64)):
+                        if isinstance(extra_coord_axes[0], numbers.Integral):
                             extra_coord_axes = [int(extra_coord_axes[0])]
                         else:
                             extra_coord_axes = list(extra_coord_axes[0]).sort()
                     else:
-                        raise ValueError("Extra coord {0} must correspond to same axes in each "
+                        raise ValueError("Extra coord {} must correspond to same axes in each "
                                          "NDCube to use it to define a plot axis.".format(
                                              axis_extra_coord))
                     # If the extra coord is a quantity, convert to the correct unit.
@@ -1034,6 +1061,7 @@ class LineAnimatorNDCubeSequence(LineAnimator):
                         # the final x-axis coord array is the same shape as the data array.
                         # This will be used in determining the correct x-axis coords for
                         # each frame of the animation.
+
                         if len(extra_coord_axes) != data_concat.ndim:
                             x_axis_coords_copy = copy.deepcopy(x_axis_coords)
                             x_axis_coords = []
@@ -1045,6 +1073,8 @@ class LineAnimatorNDCubeSequence(LineAnimator):
                                 # same as the cube's data array.
                                 # First, create shape of pre-np.tiled x-coord array for the cube.
                                 coords_reshape = np.array([1] * seq[i].data.ndim)
+                                # Convert x_axis_cube_coords to a numpy array
+                                x_axis_cube_coords = np.array(x_axis_cube_coords)
                                 coords_reshape[extra_coord_axes] = x_axis_cube_coords.shape
                                 # Then reshape x-axis array to give it the dummy dimensions.
                                 x_axis_cube_coords = x_axis_cube_coords.reshape(
@@ -1065,9 +1095,10 @@ class LineAnimatorNDCubeSequence(LineAnimator):
                         x_axis_coords = np.stack(x_axis_coords)
                 # Set x-axis label.
                 if xlabel is None:
-                    xlabel = "{0} [{1}]".format(axis_extra_coord, unit_x_axis)
+                    xlabel = f"{axis_extra_coord} [{unit_x_axis}]"
                 # Re-enter x-axis values into axis_ranges
-                axis_ranges[plot_axis_index] = x_axis_coords
+                axis_ranges[plot_axis_index] = _get_extra_coord_edges(
+                    x_axis_coords, axis=plot_axis_index)
             # Else coordinate must have been defined manually.
             else:
                 if isinstance(axis_ranges[plot_axis_index], u.Quantity):
@@ -1076,17 +1107,16 @@ class LineAnimatorNDCubeSequence(LineAnimator):
                         axis_ranges[plot_axis_index] = axis_ranges[plot_axis_index].value
                     else:
                         axis_ranges[plot_axis_index] = \
-                          axis_ranges[plot_axis_index].to(unit_x_axis).value
+                            axis_ranges[plot_axis_index].to(unit_x_axis).value
                 else:
                     if unit_x_axis is not None:
                         raise TypeError(AXES_UNIT_ERRONESLY_SET_MESSAGE)
                 if xlabel is None:
-                    xlabel = " [{0}]".format(unit_x_axis)
+                    xlabel = f" [{unit_x_axis}]"
         # Make label for y-axis.
         if ylabel is None:
-            ylabel = "Data [{0}]".format(data_unit)
-
-        super(LineAnimatorNDCubeSequence, self).__init__(
+            ylabel = f"Data [{data_unit}]"
+        super().__init__(
             data_concat, plot_axis_index=plot_axis_index, axis_ranges=axis_ranges,
             xlabel=xlabel, ylabel=ylabel, xlim=xlim, ylim=ylim, **kwargs)
 
@@ -1126,6 +1156,8 @@ class LineAnimatorCubeLikeNDCubeSequence(LineAnimator):
         same length as the axis which will provide all values for that slider.
         If None is specified for an axis then the array indices will be used
         for that axis.
+        The physical coordinates expected by axis_ranges should be an array of
+        pixel_edges.
 
     interval: `int`
         Animation interval in ms
@@ -1146,8 +1178,8 @@ class LineAnimatorCubeLikeNDCubeSequence(LineAnimator):
         The unit of y axis.
 
     Extra keywords are passed to imshow.
-
     """
+
     def __init__(self, seq, plot_axis_index=None, axis_ranges=None, unit_x_axis=None,
                  data_unit=None, xlabel=None, ylabel=None, xlim=None, ylim=None, **kwargs):
         if plot_axis_index is None:
@@ -1193,16 +1225,16 @@ class LineAnimatorCubeLikeNDCubeSequence(LineAnimator):
             # Define unit of x-axis if not supplied by user.
             if unit_x_axis is None:
                 wcs_plot_axis_index = utils.cube.data_axis_to_wcs_axis(
-                    plot_axis_index, seq[0].missing_axis)
+                    plot_axis_index, seq[0].missing_axes)
                 unit_x_axis = np.asarray(
-                    seq[0].wcs.wcs.cunit)[np.invert(seq[0].missing_axis)][wcs_plot_axis_index]
+                    seq[0].wcs.wcs.cunit)[np.invert(seq[0].missing_axes)][wcs_plot_axis_index]
             if plot_axis_index == seq._common_axis:
                 # Determine whether common axis is dependent.
                 x_axis_cube_coords = np.concatenate(
                     [cube.axis_world_coords(plot_axis_index).to(unit_x_axis).value
                      for cube in seq.data], axis=plot_axis_index)
                 dependent_axes = utils.wcs.get_dependent_data_axes(
-                    seq[0].wcs, plot_axis_index, seq[0].missing_axis)
+                    seq[0].wcs, plot_axis_index, seq[0].missing_axes)
                 if len(dependent_axes) > 1:
                     independent_axes = list(range(data_concat.ndim))
                     for i in list(dependent_axes)[::-1]:
@@ -1217,29 +1249,33 @@ class LineAnimatorCubeLikeNDCubeSequence(LineAnimator):
                     # The repeats is the inverse of dummy_reshape.
                     tile_shape = copy.deepcopy(cube_like_shape)
                     tile_shape[np.array(dependent_axes)] = 1
-                    x_axis_coords = np.tile(x_axis_cube_coords, tile_shape)
+                    x_axis_coords = _get_extra_coord_edges(
+                        np.tile(x_axis_cube_coords, tile_shape), axis=plot_axis_index)
             else:
                 # Get x-axis values from each cube and combine into a single
                 # array for axis_ranges kwargs.
                 x_axis_coords = _get_non_common_axis_x_axis_coords(seq.data, plot_axis_index,
                                                                    unit_x_axis)
-                axis_ranges[plot_axis_index] = np.concatenate(x_axis_coords, axis=seq._common_axis)
+                axis_ranges[plot_axis_index] = _get_extra_coord_edges(
+                    np.concatenate(x_axis_coords, axis=seq._common_axis), axis=plot_axis_index)
             # Set axis labels and limits, etc.
             if xlabel is None:
-                xlabel = "{0} [{1}]".format(
+                xlabel = "{} [{}]".format(
                     seq.cube_like_world_axis_physical_types[plot_axis_index], unit_x_axis)
             if ylabel is None:
-                ylabel = "Data [{0}]".format(data_unit)
-            if axis_ranges == None:
+                ylabel = f"Data [{data_unit}]"
+            if axis_ranges is None:
                 axis_ranges = [None] * data_concat.ndim
 
-        super(LineAnimatorCubeLikeNDCubeSequence, self).__init__(
+        super().__init__(
             data_concat, plot_axis_index=plot_axis_index, axis_ranges=axis_ranges,
             xlabel=xlabel, ylabel=ylabel, xlim=xlim, ylim=ylim, **kwargs)
 
 
 def _get_non_common_axis_x_axis_coords(seq_data, plot_axis_index, unit_x_axis):
-    """Get coords of an axis from NDCubes and combine into single array."""
+    """
+    Get coords of an axis from NDCubes and combine into single array.
+    """
     x_axis_coords = []
     for i, cube in enumerate(seq_data):
         # Get the x-axis coordinates for each cube.
@@ -1252,13 +1288,13 @@ def _get_non_common_axis_x_axis_coords(seq_data, plot_axis_index, unit_x_axis):
         if x_axis_cube_coords.shape != cube.data.shape:
             # Get sequence axes dependent and independent of plot_axis_index.
             dependent_axes = utils.wcs.get_dependent_data_axes(
-                cube.wcs, plot_axis_index, cube.missing_axis)
+                cube.wcs, plot_axis_index, cube.missing_axes)
             independent_axes = list(range(len(cube.dimensions)))
             for i in list(dependent_axes)[::-1]:
                 independent_axes.pop(i)
             # Expand dimensionality of x_axis_cube_coords using np.tile
             tile_shape = tuple(list(np.array(
-                cube.data.shape)[independent_axes]) + [1]*len(dependent_axes))
+                cube.data.shape)[independent_axes]) + [1] * len(dependent_axes))
             x_axis_cube_coords = np.tile(x_axis_cube_coords, tile_shape)
             # Since np.tile puts original array's dimensions as last,
             # reshape x_axis_cube_coords to cube's shape.
@@ -1289,7 +1325,6 @@ def _determine_sequence_units(cubesequence_data, unit=None):
     unit: `astropy.units.Unit`
         If input unit is not None, then the same as input.  Otherwise it is
         the unit of the first cube in the sequence.
-
     """
     # Check that the unit attribute is set of all cubes.  If not, unit_y_axis
     sequence_units = []
@@ -1313,7 +1348,7 @@ def _determine_sequence_units(cubesequence_data, unit=None):
 def _make_1D_sequence_plot(xdata, ydata, yerror, unit_y_axis, default_xlabel, kwargs):
     # Define plot settings if not set in kwargs.
     xlabel = kwargs.pop("xlabel", default_xlabel)
-    ylabel = kwargs.pop("ylabel", "Data [{0}]".format(unit_y_axis))
+    ylabel = kwargs.pop("ylabel", f"Data [{unit_y_axis}]")
     title = kwargs.pop("title", "")
     xlim = kwargs.pop("xlim", None)
     ylim = kwargs.pop("ylim", None)
@@ -1330,7 +1365,8 @@ def _make_1D_sequence_plot(xdata, ydata, yerror, unit_y_axis, default_xlabel, kw
 
 def _prep_axes_kwargs(naxis, plot_axis_indices, axes_coordinates, axes_units):
     """
-    Checks input values are correct based on number of sequence dimensions and sets defaults.
+    Checks input values are correct based on number of sequence dimensions and
+    sets defaults.
 
     Parameters
     ----------
@@ -1346,10 +1382,11 @@ def _prep_axes_kwargs(naxis, plot_axis_indices, axes_coordinates, axes_units):
 
     axes_coordinates: `None` or `list` of `None` `astropy.units.Quantity` `numpy.ndarray` `str`
         Length of list equals number of sequence axes.
+        The physical coordinates expected by axes_coordinates should be an array of
+        pixel_edges.
 
     axes_units: None or `list` of `None` `astropy.units.Unit` or `str`
         Length of list equals number of sequence axes.
-
     """
     # If plot_axis_indices, axes_coordinates, axes_units are not None and not lists,
     # convert to lists for consistent indexing behaviour.
@@ -1382,12 +1419,12 @@ def _prep_axes_kwargs(naxis, plot_axis_indices, axes_coordinates, axes_units):
         # Now axes_coordinates have been converted to a consistent convention,
         # ensure their length equals the number of sequence dimensions.
         if len(axes_coordinates) != naxis:
-            raise ValueError("length of axes_coordinates must be {0}.".format(naxis))
+            raise ValueError(f"length of axes_coordinates must be {naxis}.")
         # Ensure all elements in axes_coordinates are of correct types.
         ax_coord_types = (u.Quantity, np.ndarray, str)
         for axis_coordinate in axes_coordinates:
             if axis_coordinate is not None and not isinstance(axis_coordinate, ax_coord_types):
-                raise TypeError("axes_coordinates must be one of {0} or list of those.".format(
+                raise TypeError("axes_coordinates must be one of {} or list of those.".format(
                     [None] + list(ax_coord_types)))
     if axes_units is not None:
         if naxis > 1:
@@ -1398,7 +1435,7 @@ def _prep_axes_kwargs(naxis, plot_axis_indices, axes_coordinates, axes_units):
         # Now axes_units have been converted to a consistent convention,
         # ensure their length equals the number of sequence dimensions.
         if len(axes_units) != naxis:
-            raise ValueError("length of axes_units must be {0}.".format(naxis))
+            raise ValueError(f"length of axes_units must be {naxis}.")
         # Ensure all elements in axes_units are of correct types.
         ax_unit_types = (u.UnitBase, str)
         for axis_unit in axes_units:
