@@ -1,25 +1,25 @@
-# -*- coding: utf-8 -*-
 
 import abc
 import warnings
+import textwrap
 
 import numpy as np
 import astropy.nddata
 import astropy.units as u
-from astropy.utils.misc import InheritDocstrings
 
 from ndcube import utils
 from ndcube.ndcube_sequence import NDCubeSequence
 from ndcube.utils.wcs import wcs_ivoa_mapping
+from ndcube.utils.cube import _pixel_centers_or_edges, _get_dimension_for_pixel
 from ndcube.mixins import NDCubeSlicingMixin, NDCubePlotMixin
 
 
 __all__ = ['NDCubeABC', 'NDCubeBase', 'NDCube', 'NDCubeOrdered']
 
 
-class NDCubeMetaClass(abc.ABCMeta, InheritDocstrings):
+class NDCubeMetaClass(abc.ABCMeta):
     """
-    A metaclass that combines `abc.ABCMeta` and `~astropy.utils.misc.InheritDocstrings`.
+    A metaclass that combines `abc.ABCMeta`.
     """
 
 
@@ -77,7 +77,6 @@ class NDCubeABC(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
             reverse of the wcs axis order.
         """
 
-    # InheritDocstrings doesn't work on property methods.
     @abc.abstractproperty
     def dimensions(self):
         pass
@@ -121,14 +120,13 @@ class NDCubeABC(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
         Returns
         -------
         result: NDCube
-
         """
 
 
 class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
     """
-    Class representing N dimensional cubes.
-    Extra arguments are passed on to `~astropy.nddata.NDData`.
+    Class representing N dimensional cubes. Extra arguments are passed on to
+    `~astropy.nddata.NDData`.
 
     Parameters
     ----------
@@ -170,25 +168,31 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         Note however that it is not always possible to save the input as reference.
         Default is False.
 
-    missing_axis : `list` of `bool`
+    missing_axes : `list` of `bool`
         Designates which axes in wcs object do not have a corresponding axis is the data.
         True means axis is "missing", False means axis corresponds to a data axis.
         Ordering corresponds to the axis ordering in the WCS object, i.e. reverse of data.
         For example, say the data's y-axis corresponds to latitude and x-axis corresponds
         to wavelength.  In order the convert the y-axis to latitude the WCS must contain
         a "missing" longitude axis as longitude and latitude are not separable.
-
     """
 
     def __init__(self, data, wcs, uncertainty=None, mask=None, meta=None,
-                 unit=None, extra_coords=None, copy=False, missing_axis=None, **kwargs):
-        if missing_axis is None:
-            self.missing_axis = [False]*wcs.naxis
-        else:
-            self.missing_axis = missing_axis
+                 unit=None, extra_coords=None, copy=False, missing_axes=None, **kwargs):
+        if missing_axes is None:
+            # Ensure old missing_axis name not being used. If so, raise deprecation warning.
+            missing_axes = kwargs.get("missing_axis", None)
+            if missing_axes is None:
+                missing_axes = [False] * wcs.naxis
+            else:
+                warnings.warn(
+                    "missing_axis has been deprecated by missing_axes and "
+                    "will no longer be supported after ndcube 2.0.",
+                    DeprecationWarning)
+        self.missing_axes = missing_axes
         if data.ndim is not wcs.naxis:
             count = 0
-            for bool_ in self.missing_axis:
+            for bool_ in self.missing_axes:
                 if not bool_:
                     count += 1
             if count is not data.ndim:
@@ -197,8 +201,8 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         # Format extra coords.
         if extra_coords:
             self._extra_coords_wcs_axis = \
-              utils.cube._format_input_extra_coords_to_extra_coords_wcs_axis(
-                  extra_coords, self.missing_axis, data.shape)
+                utils.cube._format_input_extra_coords_to_extra_coords_wcs_axis(
+                    extra_coords, self.missing_axes, data.shape)
         else:
             self._extra_coords_wcs_axis = None
         # Initialize NDCube.
@@ -208,27 +212,28 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
     @property
     def dimensions(self):
         """
-        Returns a named tuple with two attributes: 'shape' gives the shape
-        of the data dimensions; 'axis_types' gives the WCS axis type of each dimension,
-        e.g. WAVE or HPLT-TAN for wavelength of helioprojected latitude.
-
+        Returns a named tuple with two attributes: 'shape' gives the shape of
+        the data dimensions; 'axis_types' gives the WCS axis type of each
+        dimension, e.g. WAVE or HPLT-TAN for wavelength of helioprojected
+        latitude.
         """
         return u.Quantity(self.data.shape, unit=u.pix)
 
     @property
     def world_axis_physical_types(self):
         """
-        Returns an iterable of strings describing the physical type for each world axis.
-
-        The strings conform to the International Virtual Observatory Alliance
-        standard, UCD1+ controlled Vocabulary.  For a description of the standard and
-        definitions of the different strings and string components,
-        see http://www.ivoa.net/documents/latest/UCDlist.html.
+        Returns an iterable of strings describing the physical type for each
+        world axis.
 
+        The strings conform to the International Virtual Observatory
+        Alliance standard, UCD1+ controlled Vocabulary.  For a
+        description of the standard and definitions of the different
+        strings and string components, see
+        http://www.ivoa.net/documents/latest/UCDlist.html.
         """
         ctype = list(self.wcs.wcs.ctype)
         axes_ctype = []
-        for i, axis in enumerate(self.missing_axis):
+        for i, axis in enumerate(self.missing_axes):
             if not axis:
                 # Find keys in wcs_ivoa_mapping dict that represent start of CTYPE.
                 # Ensure CTYPE is capitalized.
@@ -242,17 +247,25 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
                                   "Please raise an issue at "
                                   "https://github.com/sunpy/ndcube/issues citing the "
                                   "unsupported CTYPE as we'll include it: "
-                                  "CTYPE = {0}".format(ctype[i]))
-                    axis_name = "custom:{0}".format(ctype[i])
+                                  "CTYPE = {}".format(ctype[i]))
+                    axis_name = "custom:{}".format(ctype[i])
                 # If there are multiple valid keys, raise an error.
                 else:
                     raise ValueError("Non-unique CTYPE key.  Please raise an issue at "
                                      "https://github.com/sunpy/ndcube/issues citing the "
                                      "following  CTYPE and non-unique keys: "
-                                     "CTYPE = {0}; keys = {1}".format(ctype[i], keys))
+                                     "CTYPE = {}; keys = {}".format(ctype[i], keys))
                 axes_ctype.append(axis_name)
         return tuple(axes_ctype[::-1])
 
+    @property
+    def missing_axis(self):
+        warnings.warn(
+            ("The missing_axis list has been deprecated by missing_axes"
+             " and will no longer be supported after ndcube 2.0."),
+            DeprecationWarning)
+        return self.missing_axes
+
     def pixel_to_world(self, *quantity_axis_list):
         # The docstring is defined in NDDataBase
 
@@ -261,11 +274,11 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         indexed_not_as_one = []
         result = []
         quantity_index = 0
-        for i in range(len(self.missing_axis)):
-            wcs_index = self.wcs.naxis-1-i
-            # the cases where the wcs dimension was made 1 and the missing_axis is True
-            if self.missing_axis[wcs_index]:
-                list_arg.append(self.wcs.wcs.crpix[wcs_index]-1+origin)
+        for i in range(len(self.missing_axes)):
+            wcs_index = self.wcs.naxis - 1 - i
+            # the cases where the wcs dimension was made 1 and the missing_axes is True
+            if self.missing_axes[wcs_index]:
+                list_arg.append(self.wcs.wcs.crpix[wcs_index] - 1 + origin)
             else:
                 # else it is not the case where the dimension of wcs is 1.
                 list_arg.append(quantity_axis_list[quantity_index].to(u.pix).value)
@@ -287,11 +300,11 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         indexed_not_as_one = []
         result = []
         quantity_index = 0
-        for i in range(len(self.missing_axis)):
-            wcs_index = self.wcs.naxis-1-i
-            # the cases where the wcs dimension was made 1 and the missing_axis is True
-            if self.missing_axis[wcs_index]:
-                list_arg.append(self.wcs.wcs.crval[wcs_index]+1-origin)
+        for i in range(len(self.missing_axes)):
+            wcs_index = self.wcs.naxis - 1 - i
+            # the cases where the wcs dimension was made 1 and the missing_axes is True
+            if self.missing_axes[wcs_index]:
+                list_arg.append(self.wcs.wcs.crval[wcs_index] + 1 - origin)
             else:
                 # else it is not the case where the dimension of wcs is 1.
                 list_arg.append(
@@ -306,7 +319,7 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
             result.append(u.Quantity(world_to_pixel[index], unit=u.pix))
         return result[::-1]
 
-    def axis_world_coords(self, *axes):
+    def axis_world_coords(self, *axes, edges=False):
         """
         Returns WCS coordinate values of all pixels for all axes.
 
@@ -318,6 +331,11 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
             of axes for which real world coordinates are desired.
             axes=None implies all axes will be returned.
 
+        edges: `bool`
+            The edges argument helps in returning `pixel_edges`
+            instead of `pixel_values`. Default value is False,
+            which returns `pixel_values`. True return `pixel_edges`
+
         Returns
         -------
         axes_coords: `list` of `astropy.units.Quantity`
@@ -327,12 +345,12 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         -------
         >>> NDCube.all_world_coords(('lat', 'lon')) # doctest: +SKIP
         >>> NDCube.all_world_coords(2) # doctest: +SKIP
-
         """
         # Define the dimensions of the cube and the total number of axes.
         cube_dimensions = np.array(self.dimensions.value, dtype=int)
         n_dimensions = cube_dimensions.size
         world_axis_types = self.world_axis_physical_types
+
         # Determine axis numbers of user supplied axes.
         if axes == ():
             int_axes = np.arange(n_dimensions)
@@ -354,7 +372,7 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
                         int_axes[i] = utils.cube.get_axis_number_from_axis_name(
                             axis, world_axis_types)
         # Ensure user has not entered the same axis twice.
-        repeats = set([x for x in int_axes if np.where(int_axes == x)[0].size > 1])
+        repeats = {x for x in int_axes if np.where(int_axes == x)[0].size > 1}
         if repeats:
             raise ValueError("The following axes were specified more than once: {}".format(
                 ' '.join(map(str, repeats))))
@@ -363,7 +381,7 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         axes_translated = np.zeros_like(int_axes, dtype=bool)
         # Determine which axes are dependent on others.
         # Ensure the axes are in numerical order.
-        dependent_axes = [list(utils.wcs.get_dependent_data_axes(self.wcs, axis, self.missing_axis))
+        dependent_axes = [list(utils.wcs.get_dependent_data_axes(self.wcs, axis, self.missing_axes))
                           for axis in int_axes]
         n_dependent_axes = [len(da) for da in dependent_axes]
         # Iterate through each axis and perform WCS translation.
@@ -373,23 +391,25 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
                 if n_dependent_axes[i] == 1:
                     # Construct pixel quantities in each dimension letting
                     # other dimensions all have 0 pixel value.
-                    quantity_list = [
-                        u.Quantity(np.zeros(cube_dimensions[dependent_axes[i]]),
-                                   unit=u.pix)] * n_dimensions
                     # Replace array in quantity list corresponding to current axis with
                     # np.arange array.
-                    quantity_list[axis] = u.Quantity(np.arange(cube_dimensions[axis]), unit=u.pix)
+                    quantity_list = [u.Quantity(np.zeros(_get_dimension_for_pixel(
+                        cube_dimensions[dependent_axes[i]], edges)), unit=u.pix)] * n_dimensions
+                    quantity_list[axis] = u.Quantity(
+                        _pixel_centers_or_edges(
+                            cube_dimensions[axis], edges), unit=u.pix)
                 else:
                     # If the axis is dependent on another, perform
                     # translations on all dependent axes.
                     # Construct pixel quantities in each dimension letting
                     # other dimensions all have 0 pixel value.
-                    quantity_list = [u.Quantity(np.zeros(tuple(
-                        [cube_dimensions[k] for k in dependent_axes[i]])),
-                        unit=u.pix)] * n_dimensions
                     # Construct orthogonal pixel index arrays for dependent axes.
+                    quantity_list = [u.Quantity(np.zeros(tuple(
+                        [_get_dimension_for_pixel(cube_dimensions[k], edges) for k in dependent_axes[i]])),
+                        unit=u.pix)] * n_dimensions
+
                     dependent_pixel_quantities = np.meshgrid(
-                        *[np.arange(cube_dimensions[k]) * u.pix
+                        *[_pixel_centers_or_edges(cube_dimensions[k], edges) * u.pix
                           for k in dependent_axes[i]], indexing="ij")
                     for k, axis in enumerate(dependent_axes[i]):
                         quantity_list[axis] = dependent_pixel_quantities[k]
@@ -415,14 +435,14 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         Dictionary of extra coords where each key is the name of an extra
         coordinate supplied by user during instantiation of the NDCube.
 
-        The value of each key is itself a dictionary with the following keys:
-          | 'axis': `int`
-          |     The number of the data axis to which the extra coordinate corresponds.
-          | 'value': `astropy.units.Quantity` or array-like
-          |     The value of the extra coordinate at each pixel/array element along the
-          |     corresponding axis (given by the 'axis' key, above).  Note this means
-          |     that the length of 'value' must be equal to the length of the data axis
-          |     to which is corresponds.
+        The value of each key is itself a dictionary with the following
+        keys:   | 'axis': `int`   |     The number of the data axis to
+        which the extra coordinate corresponds.   | 'value':
+        `astropy.units.Quantity` or array-like   |     The value of the
+        extra coordinate at each pixel/array element along the   |
+        corresponding axis (given by the 'axis' key, above).  Note this
+        means   |     that the length of 'value' must be equal to the
+        length of the data axis   |     to which is corresponds.
         """
 
         if not self._extra_coords_wcs_axis:
@@ -433,7 +453,7 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
                 result[key] = {
                     "axis": utils.cube.wcs_axis_to_data_axis(
                         self._extra_coords_wcs_axis[key]["wcs axis"],
-                        self.missing_axis),
+                        self.missing_axes),
                     "value": self._extra_coords_wcs_axis[key]["value"]}
         return result
 
@@ -446,7 +466,8 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         # This part of the code will be removed in version 2.0
         if interval_widths:
             warnings.warn(
-                "interval_widths will be removed from the API in version 2.0, please use upper_corner argument.")
+                "interval_widths will be removed from the API in version 2.0"
+                ", please use upper_corner argument.")
             if upper_corner:
                 raise ValueError("Only one of interval_widths or upper_corner "
                                  "can be set. Recommend using upper_corner as "
@@ -479,20 +500,21 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         all_world_corners_grid = np.meshgrid(
             *[u.Quantity([lower_corner[i], upper_corner[i]], unit=lower_corner[i].unit).value
               for i in range(self.data.ndim)])
-        all_world_corners = [all_world_corners_grid[i].flatten()*lower_corner[i].unit
+        all_world_corners = [all_world_corners_grid[i].flatten() * lower_corner[i].unit
                              for i in range(n_dim)]
         # Convert to pixel coordinates
         all_pix_corners = self.world_to_pixel(*all_world_corners)
         # Derive slicing item with which to slice NDCube.
         # Be sure to round down min pixel and round up + 1 the max pixel.
         item = tuple([slice(int(np.clip(axis_pixels.value.min(), 0, None)),
-                            int(np.ceil(axis_pixels.value.max()))+1)
+                            int(np.ceil(axis_pixels.value.max())) + 1)
                       for axis_pixels in all_pix_corners])
         return self[item]
 
     def crop_by_extra_coord(self, coord_name, min_coord_value, max_coord_value):
         """
-        Crops an NDCube given a minimum value and interval width along an extra coord.
+        Crops an NDCube given a minimum value and interval width along an extra
+        coord.
 
         Parameters
         ----------
@@ -510,7 +532,6 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         Returns
         -------
         result: `ndcube.NDCube`
-
         """
         if not isinstance(coord_name, str):
             raise TypeError("The API for this function has changed. "
@@ -523,25 +544,26 @@ class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
         w = np.logical_and(extra_coord_values >= min_coord_value,
                            extra_coord_values < max_coord_value)
         w = np.arange(len(extra_coord_values))[w]
-        item = [slice(None)]*len(self.dimensions)
-        item[extra_coord_dict["axis"]] = slice(w[0], w[-1]+1)
+        item = [slice(None)] * len(self.dimensions)
+        item[extra_coord_dict["axis"]] = slice(w[0], w[-1] + 1)
         return self[tuple(item)]
 
+    def __str__(self):
+        return textwrap.dedent(f"""\
+                NDCube
+                ---------------------
+                {{wcs}}
+                ---------------------
+                Length of NDCube: {self.dimensions}
+                Axis Types of NDCube: {self.world_axis_physical_types}""").format(wcs=str(self.wcs))
+
     def __repr__(self):
-        return (
-            """NDCube
----------------------
-{wcs}
----------------------
-Length of NDCube: {lengthNDCube}
-Axis Types of NDCube: {axis_type}
-""".format(wcs=self.wcs.__repr__(), lengthNDCube=self.dimensions,
-           axis_type=self.world_axis_physical_types))
+        return f"{object.__repr__(self)}\n{str(self)}"
 
     def explode_along_axis(self, axis):
         """
-        Separates slices of NDCubes along a given cube axis into a NDCubeSequence
-        of (N-1)DCubes.
+        Separates slices of NDCubes along a given cube axis into a
+        NDCubeSequence of (N-1)DCubes.
 
         Parameters
         ----------
@@ -551,7 +573,6 @@ Axis Types of NDCube: {axis_type}
         Returns
         -------
         result : `ndcube_sequence.NDCubeSequence`
-
         """
         # If axis is -ve then calculate the axis from the length of the dimensions of one cube
         if axis < 0:
@@ -573,18 +594,18 @@ Axis Types of NDCube: {axis_type}
         # Creating a new NDCubeSequence with the result_cubes and common axis as axis
         return NDCubeSequence(result_cubes, common_axis=axis, meta=self.meta)
 
+
 class NDCube(NDCubeBase, NDCubePlotMixin, astropy.nddata.NDArithmeticMixin):
     pass
 
 
 class NDCubeOrdered(NDCube):
     """
-    Class representing N dimensional cubes with oriented WCS.
-    Extra arguments are passed on to NDData's init.
-    The order is TIME, SPECTRAL, SOLAR-x, SOLAR-Y and any other dimension.
-    For example, in an x, y, t cube the order would be (t,x,y) and in a
-    lambda, t, y cube the order will be (t, lambda, y).
-    Extra arguments are passed on to NDData's init.
+    Class representing N dimensional cubes with oriented WCS. Extra arguments
+    are passed on to NDData's init. The order is TIME, SPECTRAL, SOLAR-x,
+    SOLAR-Y and any other dimension. For example, in an x, y, t cube the order
+    would be (t,x,y) and in a lambda, t, y cube the order will be (t, lambda,
+    y). Extra arguments are passed on to NDData's init.
 
     Parameters
     ----------
@@ -625,7 +646,7 @@ class NDCubeOrdered(NDCube):
     """
 
     def __init__(self, data, wcs, uncertainty=None, mask=None, meta=None,
-                 unit=None, extra_coords=None, copy=False, missing_axis=None, **kwargs):
+                 unit=None, extra_coords=None, copy=False, missing_axes=None, **kwargs):
         axtypes = list(wcs.wcs.ctype)[::-1]
         array_order = utils.cube.select_order(axtypes)
         result_data = data.transpose(array_order)
@@ -649,4 +670,4 @@ class NDCubeOrdered(NDCube):
         super().__init__(result_data, result_wcs, uncertainty=result_uncertainty,
                          mask=result_mask, meta=meta, unit=unit,
                          extra_coords=reordered_extra_coords,
-                         copy=copy, missing_axis=missing_axis, **kwargs)
+                         copy=copy, missing_axes=missing_axes, **kwargs)
